import 'package:test/test.dart';

import 'package:moving_average/moving_average.dart';

void main() {
  group('Mean (average) tests', () {
    test('Mean of [] is NaN', () {
      final List<num> input = [];
      final result = mean(input);
      expect(result, isNaN);
    });

    test('Mean of [1] is 1', () {
      final input = [1];
      final result = mean(input);
      final expected = 1.0;
      expect(result, expected);
    });

    test('Mean of [1..5] is 3.0', () {
      final input = [1, 2, 3, 4, 5];
      final result = mean(input);
      final expected = 3.0;
      expect(result, expected);
    });
  });

  group('Weighted Mean tests', () {
    test('Weighted Mean of [] is NaN', () {
      final List<num> input = [];
      final result = weightedMean(input);
      expect(result, isNaN);
    });

    test('Weighted Mean of [1..5] is 3.6666666666666665', () {
      final input = [1, 2, 3, 4, 5];
      final expected = 3.6666666666666665;
      final result = weightedMean(input);
      expect(result, expected);
    });
  });

  group('Sliding Window tests', () {
    test('Window of [], size 2', () {
      final List<num> input = [];
      final result = window(input, 2, includePartial: false).toList(growable: false);
      final expected = [];
      expect(result, expected);
    });

    test('Window of [1], size 2, no partial', () {
      final input = [1];
      final result = window(input, 2, includePartial: false).toList(growable: false);
      final expected = [];
      expect(result, expected);
    });

    test('Window of [1], size 2, partial', () {
      final input = [1];
      final result = window(input, 2, includePartial: true).toList(growable: false);
      final expected = [[1]];
      expect(result, expected);
    });

    test('Window of [1..5], size 2, no partial', () {
      final input = [1, 2, 3, 4, 5];
      final result = window(input, 2, includePartial: false).toList(growable: false);
      final expected = [
        [1, 2],
        [2, 3],
        [3, 4],
        [4, 5]
      ];
      expect(result, expected);
    });

    test('Window of [1..5], size 2, partial', () {
      final input = [1, 2, 3, 4, 5];
      final result = window(input, 2, includePartial: true).toList(growable: false);
      final expected = [
        [1],
        [1, 2],
        [2, 3],
        [3, 4],
        [4, 5]
      ];
      expect(result, expected);
    });
  });

  group('Moving Average (mean) tests', () {
    test('Moving Average of [], size 2, no partial is []', () {
      final List<num> input = [];
      final result = movingAverage(input, 2, includePartial: false);
      final expected = [];
      expect(result, expected);
    });

    test('Moving Average of [1..5], size 2, no partial', () {
      final input = [1, 2, 3, 4, 5];
      final result = movingAverage(input, 2, includePartial: false);
      final expected = [1.5, 2.5, 3.5, 4.5];
      expect(result, expected);
    });
  });

  group('Weighted Moving Average tests', () {
    test('Weighted Moving Average of [], size 2, no partial is []', () {
      final List<num> input = [];
      final result = weightedMovingAverage(input, 2, includePartial: false);
      final expected = [];
      expect(result, expected);
    });

    test('Weighted Moving Average of [1], size 2, no partial', () {
      final input = [1];
      final result = weightedMovingAverage(input, 2, includePartial: false);
      final expected = [];
      expect(result, expected);
    });

    test('Weighted Moving Average of [1], size 2, partial', () {
      final input = [1];
      final result = weightedMovingAverage(input, 2, includePartial: true);
      final expected = [1.0];
      expect(result, expected);
    });

    test('Weighted Moving Average of [1..5], size 2, no partial', () {
      final input = [1, 2, 3, 4, 5];
      final result = weightedMovingAverage(input, 2, includePartial: false);
      final expected = [
        1.6666666666666667,
        2.6666666666666665,
        3.6666666666666665,
        4.666666666666667
      ];
      expect(result, expected);
    });

    test('Weighted Moving Average of [1..5], size 2, partial', () {
      final input = [1, 2, 3, 4, 5];
      final result = weightedMovingAverage(input, 2, includePartial: true);
      final expected = [
        1.0,
        1.6666666666666667,
        2.6666666666666665,
        3.6666666666666665,
        4.666666666666667
      ];
      expect(result, expected);
    });
  });
}
